package sda.jdbc.intro.v1;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;

/**
 * Handle database communication. There are two
 * distinct internal states <q>disconnected</q> and <q>connected</q>, see
 * {@link #isConnected()}. These two states may be toggled by invoking
 * {@link #connect()} and {@link #disconnect()} respectively.
 * 
 * The following snippet illustrates the intended usage:
 * <pre>   public static void main(String[] args) {
      final PersistenceHandler ph = new PersistenceHandler();
      if (ph.connect()) {
         if (!ph.add("Jim", "jim@foo.com")) {
            System.err.println("Insert Error:" + ph.getErrorMessage());
         }
      } else {
         System.err.println("Connect error:" + ph.getErrorMessage());
      }
   }</pre>
 * 
 * @author goik
 */
public class PersistenceHandler {

	Connection conn = null;
	Statement stmt = null;

	String errorMessage = null;
	
	/**
	 * Instance in <q>disconnected</q> state. See {@link #isConnected()}
	 */
	public PersistenceHandler() {/* only present here to supply javadoc comment */}

   /**
    * Inserting a (name, email) record into the database server. In case of
    * errors corresponding messages may subsequently be retrieved by calling
    * {@link #getErrorMessage()}.
    * 
    * <dt><b>Precondition:</b></dt> <dd>must be in
    * <q>connected</q> state, see {@link #isConnected()}</dd>
    * 
    * @param name
    *           A person's name
    * @param email
    *           A person's email address
    * 
    * @return true if the current data record has been successfully inserted
    *         into the database server. false in case of error(s).
    */
	public boolean add(final String name, final String email){
		final String sql = "INSERT INTO Person VALUES('" + name + "', '" + 
				email + "')"; 
		try {
			stmt.executeUpdate(sql);
			return true;
		} catch (SQLException e) {
			errorMessage = "Unable to execute '" + sql + "': '" + e.getMessage() + "'";
			return false;
		}
	}
	
   /**
    * Retrieving error messages in case a call to {@link #add(String, String)},
    * {@link #connect()}, or {@link #disconnect()} yields an error.
    * 
    * @return the error explanation corresponding to the latest failed
    *         operation, null if no error yet occurred.
    */
	public String getErrorMessage() {
		return errorMessage;
	}
	
	/**
	 * Open a connection to a database server.
	 * 
    * <dt><b>Precondition:</b><dd>
    *  <dd>must be in <q>disconnected</q> state, see {@link #isConnected()}</dd>
    *  
    * <dt><b>Precondition:</b><dd>
    *  <dd>The following properties must be set:
    *  <pre>PersistenceHandler.jdbcUrl=jdbc:mysql://localhost:3306/hdm
PersistenceHandler.password=XYZ
PersistenceHandler.username=foo</pre>
    *  </dd>
	 * 
	 * @return true if connecting was successful
	 */
	public boolean connect () {
		try {
			conn = DriverManager.getConnection(
					DbProps.getString("PersistenceHandler.jdbcUrl"),
					DbProps.getString("PersistenceHandler.username"),
					DbProps.getString("PersistenceHandler.password"));
			try {
				stmt = conn.createStatement();
				return true; 
			} catch (SQLException e) {
				errorMessage = "Connection opened but Statement creation failed:\"" + e.getMessage() + "\"."; 
				try {
					conn.close();
				} catch (SQLException ee) {
					errorMessage += "Closing connection failed:\"" + e.getMessage() + "\".";
				}
				conn = null;
			}

		} catch (SQLException e) {
			errorMessage = "Unable to open connection:\"" + e.getMessage() + "\".";
		}
		return false;
	}
	
	/**
	 * Close a connection to a database server and clean up JDBC related resources
	 * 
	 * Error messages in case of failure may subsequently be retrieved by
	 * calling {@link #getErrorMessage()}.
	 * 
	 * <dt><b>Precondition:</b></dt>
	 * <dd>must be in <q>connected</q> state, see {@link #isConnected()}</dd>
	 * 
	 * @return true if disconnecting was successful, false in case error(s) occur.
	 */
	public boolean disconnect() {
		boolean resultStatus = true;
		final StringBuffer messageCollector = new StringBuffer();
		try {
			stmt.close();
		} catch (SQLException e) {
			resultStatus = false;
			messageCollector.append("Unable to close Statement:\"" + e.getMessage() + "\".");
		}
		stmt = null;
		try {
			conn.close();
		} catch (SQLException e) {
			resultStatus = false;
			messageCollector.append("Unable to close connection:\"" + e.getMessage() + "\".");
		}
		conn = null;
		if (!resultStatus) {
			errorMessage = messageCollector.toString();
		}
		return resultStatus;
	}
	
	/**
	 * An instance can either be in <q>connected</q> or <q>disconnected</q> state. The
	 * state can be toggled by invoking {@link #connect()} or
	 * {@link #disconnect()} respectively.
	 * 
	 * @return true if connected, false otherwise
	 */
	public boolean isConnected() {
		return null != conn;
	}
}